@@ -2,11 +2,33 @@ from __future__ import unicode_literals
 
 from copy import deepcopy
 
-from django.db import models
+import django
 from django.core.exceptions import FieldError
+from django.db import models
+from django.db.models.fields.files import FileDescriptor
 from django.db.models.query_utils import DeferredAttribute
 
 
+class DescriptorMixin(object):
+    tracker_instance = None
+
+    def __get__(self, instance, owner):
+        if instance is None:
+            return self
+        was_deferred = False
+        field_name = self._get_field_name()
+        if field_name in instance._deferred_fields:
+            instance._deferred_fields.remove(field_name)
+            was_deferred = True
+        value = super(DescriptorMixin, self).__get__(instance, owner)
+        if was_deferred:
+            self.tracker_instance.saved_data[field_name] = deepcopy(value)
+        return value
+
+    def _get_field_name(self):
+        return self.field_name
+
+
 class FieldInstanceTracker(object):
     def __init__(self, instance, fields, field_map):
         self.instance = instance
@@ -62,33 +84,48 @@ class FieldInstanceTracker(object):
         )
 
     def init_deferred_fields(self):
-        self.instance._deferred_fields = []
+        self.instance._deferred_fields = set()
         if hasattr(self.instance, '_deferred') and not self.instance._deferred:
             return
 
-        class DeferredAttributeTracker(DeferredAttribute):
-            def __get__(field, instance, owner):
-                data = instance.__dict__
-                if data.get(field.field_name, field) is field:
-                    instance._deferred_fields.remove(field.field_name)
-                    value = super(DeferredAttributeTracker, field).__get__(
-                        instance, owner)
-                    self.saved_data[field.field_name] = deepcopy(value)
-                return data[field.field_name]
+        class DeferredAttributeTracker(DescriptorMixin, DeferredAttribute):
+            tracker_instance = self
 
-        for field in self.fields:
-            field_obj = self.instance.__class__.__dict__.get(field)
-            if isinstance(field_obj, DeferredAttribute):
-                self.instance._deferred_fields.append(field)
+        class FileDescriptorTracker(DescriptorMixin, FileDescriptor):
+            tracker_instance = self
 
-                # Django 1.4
-                model = None
-                if hasattr(field_obj, 'model_ref'):
-                    model = field_obj.model_ref()
+            def _get_field_name(self):
+                return self.field.name
 
-                field_tracker = DeferredAttributeTracker(
-                    field_obj.field_name, model)
-                setattr(self.instance.__class__, field, field_tracker)
+        if django.VERSION >= (1, 8):
+            self.instance._deferred_fields = self.instance.get_deferred_fields()
+            for field in self.instance._deferred_fields:
+                if django.VERSION >= (1, 10):
+                    field_obj = getattr(self.instance.__class__, field)
+                else:
+                    field_obj = self.instance.__class__.__dict__.get(field)
+                if isinstance(field_obj, FileDescriptor):
+                    field_tracker = FileDescriptorTracker(field_obj.field)
+                    setattr(self.instance.__class__, field, field_tracker)
+                else:
+                    field_tracker = DeferredAttributeTracker(
+                        field_obj.field_name, None)
+                    setattr(self.instance.__class__, field, field_tracker)
+        else:
+            for field in self.fields:
+                field_obj = self.instance.__class__.__dict__.get(field)
+                if isinstance(field_obj, DeferredAttribute):
+                    self.instance._deferred_fields.add(field)
+
+                    # Django 1.4
+                    if django.VERSION >= (1, 5):
+                        model = None
+                    else:
+                        model = field_obj.model_ref()
+
+                    field_tracker = DeferredAttributeTracker(
+                        field_obj.field_name, model)
+                    setattr(self.instance.__class__, field, field_tracker)
 
 
 class FieldTracker(object):
